
#ifndef __LIBADWIN_H__
#define __LIBADWIN_H__

#include <sys/cdefs.h>

__BEGIN_DECLS

#include <sys/types.h>

// if 0, SETMUTEX reloads the configuration file, and creates the 
// shared memory region (kernel 2.6)
extern int initialized;
#define SEM_KEY 0x41447769 // This is ADwi in Hex :-)

#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
#else
/* according to X/OPEN we have to define it ourselves */
union semun {
	int val;                  /* value for SETVAL */
	struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
	unsigned short *array;    /* array for GETALL, SETALL */
	/* Linux specific part: */
	struct seminfo *__buf;    /* buffer for IPC_INFO */
};
#endif

/*
 * booting and process handling
 */
extern void Set_DeviceNo(int Number);
extern void Boot(const char *Filename);
extern long adwin_boot (const char *filename, long memsize, int deviceno);
extern void Load_Process(const char * Filename);
extern int adwin_process_load (const char * filename, int deviceno);
extern void Start_Process (long ProcessNo);
extern int adwin_process_start (int processno, int deviceno);
extern void Stop_Process(long ProcessNo);
extern int adwin_process_stop (int processno, int deviceno);
extern void Clear_Process (long ProcessNo);
extern int adwin_process_clear (int processno, int deviceno);
extern long Process_Status (long ProcessNo);
extern int adwin_process_status (int processno, int deviceno);

/*
 * parameter access
 */
extern long Set_Globaldelay (long ProcessNo, long Value);
extern int adwin_set_globaldelay (int processno, int32_t value, int deviceno);
extern void Set_Par (long Index, long Value);
extern int adwin_set_par (int Index, int Value, int deviceno);
extern void Set_FPar(long Index, float Value);
extern int adwin_set_fpar (int Index, float Value, int deviceno);

extern long Get_Globaldelay (long ProcessNo);
extern int adwin_get_globaldelay (int processno, int deviceno);
extern long Get_Par(long Index);
extern int adwin_get_par (int par, int deviceno);
extern float Get_FPar(long Index);
extern float adwin_get_fpar (int par, int deviceno);

extern void Get_Par_All(long *Array);
extern int adwin_get_par_all (int32_t *array, int deviceno);
extern void Get_FPar_All(float *Array);
extern int adwin_get_fpar_all (float *array, int deviceno);
extern void Get_FPar_All_Double (double *Array);
extern int adwin_get_fpar_all_double (double *array, int deviceno);

extern void Get_Par_Block (long *Array, long StartIndex, long Count);
extern int adwin_get_par_block (int first, int count, int32_t *array, int deviceno);
extern void Get_FPar_Block (float *Array, long StartIndex, long Count);
extern int adwin_get_fpar_block (int first, int count, float *array, int deviceno);
extern void Get_FPar_Block_Double(double *Array, long StartIndex, long Count);
extern int adwin_get_fpar_block_double (int first, int count, double *array, int deviceno);

/*
 * data access
 */

extern int adwin_get_data_int16 (int datano, long first, long count, int16_t *dest, int deviceno);
extern void GetData_Long(long DataNo, long *Data, long StartIndex, long Count);
extern int adwin_get_data_int32 (int datano, long first, long count, int32_t *dest, int deviceno);
extern int adwin_get_data_int64 (int datano, long first, long count, int64_t *dest, int deviceno);
extern void GetData_Float(long DataNo, float *Data, long StartIndex, long Count);
extern int adwin_get_data_float (int datano, long first, long count, float *dest, int deviceno);
extern void GetData_Double(long DataNo, double *Data, long StartIndex, long Count);
extern int adwin_get_data_double (int datano, long first, long count, double *dest, int deviceno);

extern int adwin_set_data_int16 (int datano, long first, long count, int16_t *src, int deviceno);
extern void SetData_Long(long DataNo, long *Data, long StartIndex, long Count);
extern int adwin_set_data_int32 (int datano, long first, long count, int32_t *src, int deviceno);
extern int adwin_set_data_int64 (int datano, long first, long count, int64_t *src, int deviceno);
extern void SetData_Float(long DataNo, float *Data, long StartIndex, long Count);
extern int adwin_set_data_float (int datano, long first, long count, float *src, int deviceno);
extern void SetData_Double(long DataNo, double *Data, long StartIndex, long Count);
extern int adwin_set_data_double (int datano, long first, long count, double *src, int deviceno);

extern int adwin_clear_data (int datano, int deviceno);

extern long Data_Length (long DataNo);
extern int adwin_data_length (int datano, int deviceno);
extern void Data2File (char *Filename, long DataNo, long Starindex, long Count, int Mode);
extern int adwin_data2file (char *filename, int datano, long first, long count, int mode, int deviceno);
//Get_Data_Type nicht vorhanden in ActiveX, deshalb undokumentiert!
extern int adwin_get_data_type(int datano, int deviceno);

/*
 * fifo access
 */
extern void Fifo_Clear (long FifoNo);
extern int adwin_fifo_clear (int fifono, int deviceno);
extern long Fifo_Empty (long FifoNo);
extern int adwin_fifo_empty (int fifono, int deviceno);
extern long Fifo_Full (long FifoNo);
extern int adwin_fifo_full (int fifono, int deviceno);

extern int adwin_set_fifo_int16 (int fifono, long count, int16_t *src, int deviceno);
extern void SetFifo_Long (long FifoNo, long *Data, long Count);
extern int adwin_set_fifo_int32 (int fifono, long count, int32_t *src, int deviceno);
extern int adwin_set_fifo_int64 (int fifono, long count, int64_t *src, int deviceno);
extern void SetFifo_Float (long FifoNo, float *Data, long Count);
extern int adwin_set_fifo_float (int fifono, long count, float *src, int deviceno);
extern void SetFifo_Double (long FifoNo, double *Data, long Count);
extern int adwin_set_fifo_double (int fifono, long count, double *src,  int deviceno);

extern int adwin_get_fifo_int16 (int fifono, long count, int16_t *dest, int deviceno);
extern void GetFifo_Long (long FifoNo, long *Data, long Count);
extern int adwin_get_fifo_int32 (int fifono, long count, int32_t *dest, int deviceno);
extern int adwin_get_fifo_int64 (int fifono, long count, int64_t *dest, int deviceno);
extern void GetFifo_Float (long FifoNo, float *Data, long Count);
extern int adwin_get_fifo_float (int fifono, long count, float *dest, int deviceno);
extern void GetFifo_Double (long FifoNo, double *Data, long Count);
extern int adwin_get_fifo_double (int fifono, long count, double *dest, int deviceno);

/*
 * string functions
 */
extern long GetData_String (long DataNo, char *Data, long MaxCount);
extern int adwin_get_data_string (int datano, long count, char *dest, int deviceno);
extern void SetData_String (long DataNo, char *Data);
extern int adwin_set_data_string (int datano, const char *src, int deviceno);
extern long String_Length (long DataNo);
extern long adwin_strlen (int datano, int deviceno);

/*
 * misc functions
 */
#define MEM_INTERNAL_PM		1
#define MEM_INTERNAL_DM		3
#define MEM_EXTERNAL_DRAM	4

extern long Free_Mem (long Mem_Spec);
extern long adwin_mem (int memid, int deviceno);

extern long Workload ();
extern int adwin_workload (int deviceno);
extern long Test_Version ();
extern int adwin_test_version (int deviceno);
extern long Processor_Type ();
extern long adwin_processor_type (int deviceno);

extern long Get_Last_Error ();
extern char* Get_Last_Error_Text (long Errno);

/*
 * Linux only !
 * A nonzero @err argument indicates an error.
 * Include "libadwin/errno.h as well and use
 * adwin_strerror(err) to get an error description.
 */
extern long __adwin_mem (int memid, int *err, int deviceno);

extern int __adwin_get_globaldelay (int processno, int *err, int deviceno);
extern int __adwin_get_par (int index, int *err, int deviceno);
extern float __adwin_get_fpar (int index, int *err, int deviceno);

extern int __adwin_fifo_empty (int fifono, int *err, int deviceno);
extern int __adwin_fifo_full (int fifono, int *err, int deviceno);

extern int __adwin_data_length (int datano, int *err, int deviceno);


/******************************************************************************
 *
 * Windows-compatible alias names for functions
 * We do NOT encourage you to use them!
 *
 * NOTE:
 * gcc 3.1 allows the additional __attribute__ ((deprecated)) ...
 */
#ifdef WITH_ALIAS

#define weak_alias(name, aliasname) \
	extern __typeof (name) aliasname __attribute__ ((weak, alias (#name)))

weak_alias(adwin_clear_data, Data_Clear);

#endif /* WITH_ALIAS */

__END_DECLS



/*****************************************************************************
DLL-Funktionsaufrufe Deklaration
*/

extern long ADboot(const char *Dateiname, short Linkadresse, long iboardsize, short msgbox);
extern long ADGetErrorCode(void);
extern long ADGetErrorText(long ErrCode, char *pText, long len);
extern short ADSetLanguage(short Lang);
extern short ADBload(const char *dateiname,short Linkaddresse,short msgbox);  //Load_Process
extern short ADB_Start(short np, short nadr);  //Start_process
extern short Get_List(short *tarr, short typ, short nadc, short proc, long il, short nadr);
extern short Set_List(short *tarr, short typ, short nadc, long il, short nadr);
extern short Start(short np, short nadr);  //Start von Standardprozessen
extern short Stop(short np, short nadr);  //Stop von Standardprozessen
extern short ADB_Stop(short np, short nadr);  // Stop_Process
extern short Set_ADBPar(short np, long iw, short nadr);  // Set_Par
extern short Set_ADBFPar(short np, float fw, short nadr); // Set_FPar
extern long Get_ADBPar(short np, short nadr);  //Get_Par
extern float Get_ADBFPar(short np,short nadr); // Get_FPar
extern short Get_Data(void *tarr,short typ, short nadc, long lstart, long il ,short nadr);
 	/*  typ = 2 fr long
	  typ = 5 fr float
	  typ = 6 fr double */

extern short Set_Data(void *tarr,short typ, short nadc, long lstart, long il ,short nadr);
 	/*  typ = 2 fr long
	  typ = 5 fr float
	  typ = 6 fr double */

extern short SaveFast(const char *dateiname,short Datanr,long startindex,long anzahldaten,short anhaengen,short linkaddresse);  // Data2File

extern short Get_Fifo(void *tarr,short typ, short nadc, long il ,short nadr);
	/*  typ = 2 fr long
	  typ = 5 fr float
	  typ = 6 fr double */

extern short Set_Fifo(void *tarr,short typ, short nadc, long il ,short nadr);
	/*  typ = 2 fr long
	  typ = 5 fr float
	  typ = 6 fr double */

extern long Get_Fifo_Count(short nadc, short nadr );  // Fifo_Full
extern long Get_Fifo_Empty(short nadc, short nadr );  // Fifo_Empty
extern short Clear_Fifo(short nadc, short nadr );  //Fifo_Clear


unsigned short Get_ADC (short nadc, short nadr); //ADC
short Set_DAC (short ndac, short iw, short nadr);  //SetDAC


extern long Get_Digin(short nadr);  //DigIn
extern short Set_Digout(short iw, short nadr);  //SetDigOut
extern long Get_Digout(short nadr);
extern short AD_Workload(long Priority, short Linkaddresse);  //Workload
extern short ADTest_Version(short Linkaddresse,short msgbox);  // Test_Version
extern long AD_Memory_all(short typ,short Linkaddresse);  // Free_Mem
extern long AD_Memory(short Linkaddresse);
// ADNet_Connect und / Disconnect bleibt draussen laut SJ am 8.5.03
extern short ADProzessorTyp(short Linkaddresse);
// muss aus Labview Vis entfernt werden!(wegen  Namenskonflikt) extern long Clear_Process (short proc_nr , short nadr);
extern short Get_ADBPar_All(short np,short anzahl,long *pData, short nadr); //Get_Par_Block
extern short Get_ADBFPar_All(short np,short anzahl,float *pData,short nadr); //Get_FPar_Block
extern long GetDataLength(short nr,short nadr);


/* vorerst nicht:
short Get_Known_DeviceNo(long *Devices, long Count_Devices);
DLL: short Get_Known_Deviceno(short *devices, long *count_devices);

long Get_Known_USB_SerialNo(long *SerialNo);
DLL: long Get_Known_USB_SerialNo(long *serialno);

int AD_Debug_On(BSTR * Filename, long Size);
DLL: int adwin_debug_mode_on(const char *dateiname, long size);

void AD_Debug_Off();
DLL: int adwin_debug_mode_off();
*/


extern long Get_Data_String(char *Data, long MaxCount,short DataNo, short nadr);
extern long Get_Data_String_Length(long DataNo, short nadr);
extern long Set_Data_String(const char *Data, short DataNo, short nadr);
extern short Get_Data_packed(void *tarr, short typ, short nadc, long lstart, long il ,short nadr);
	/*   typ = 1 fr short
	     typ = 2 fr long
	     typ = 5 fr float
	     typ = 6 fr double  */

#endif /* __LIBADWIN_H__ */
